package com.usky.oa.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.usky.common.core.bean.CommonPage;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.security.utils.SecurityUtils;
import com.usky.oa.domain.OaDocument;
import com.usky.oa.domain.OaFormDefinition;
import com.usky.oa.domain.OaJbdDocument;
import com.usky.oa.domain.OaQjdDocument;
import com.usky.oa.mapper.OaDocumentMapper;
import com.usky.oa.mapper.OaFormDefinitionMapper;
import com.usky.oa.mapper.OaJbdDocumentMapper;
import com.usky.oa.mapper.OaQjdDocumentMapper;
import com.usky.oa.service.OaDocumentService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.oa.service.OaJbdDocumentService;
import com.usky.oa.service.OaQjdDocumentService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 单据总表 服务实现类
 * </p>
 *
 * @author fu
 * @since 2024-12-26
 */
@Service
public class OaDocumentServiceImpl extends AbstractCrudService<OaDocumentMapper, OaDocument> implements OaDocumentService {

    @Autowired
    private OaDocumentMapper oaDocumentMapper;

    @Autowired
    private OaQjdDocumentMapper oaQjdDocumentMapper;

    @Autowired
    private OaQjdDocumentService oaQjdDocumentService;

    @Autowired
    private OaJbdDocumentService oaJbdDocumentService;

    @Autowired
    private OaJbdDocumentMapper oaJbdDocumentMapper;

    @Autowired
    private OaDocumentMapper documentMapper;

    @Autowired
    private OaFormDefinitionMapper oaFormDefinitionMapper;

    private static final String TABLE_HEAD = "oa_";
    private static final String TABLE_SUFFIX = "_document";

    // 我的申请
    @Override
    public CommonPage<OaDocument> myApplication(String docNo, String type, Integer status, String startTime, String endTime, Integer pageNum, Integer pageSize) {
        Long userId = SecurityUtils.getUserId();
        Integer tenantId = SecurityUtils.getTenantId();

        LambdaQueryWrapper<OaDocument> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(OaDocument::getTenantId, tenantId).eq(OaDocument::getProposer, userId)
                .like(StringUtils.isNotBlank(docNo), OaDocument::getDocNo, docNo)
                .eq(StringUtils.isNotBlank(type), OaDocument::getType, type)
                .eq(status != null, OaDocument::getDocStatus, status)
                .between(StringUtils.isNotBlank(startTime) && StringUtils.isNotBlank(endTime), OaDocument::getCreateTime, startTime, endTime)
                .orderByDesc(OaDocument::getId);

        return ToCommonPage(page(new Page<>(pageNum, pageSize), wrapper));
    }

    // 单据详情
    @Override
    public Object documentDetails(String docNo) {
        Object documentDetails = null;
        String type = getFormSign(docNo);

        switch (type) {
            case "QJD":
                documentDetails = oaQjdDocumentMapper.selectOne(Wrappers.lambdaQuery(OaQjdDocument.class).eq(OaQjdDocument::getDocNo, docNo));
                break;
            case "JBD":
                documentDetails = oaJbdDocumentMapper.selectOne(Wrappers.lambdaQuery(OaJbdDocument.class).eq(OaJbdDocument::getDocNo, docNo));
                break;
        }

        return documentDetails;
    }

    // 删除单据
    @Override
    public void delDocument(String docNo) {
        String type = getFormSign(docNo);
        switch (type) {
            case "QJD":
                oaQjdDocumentMapper.delete(Wrappers.lambdaQuery(OaQjdDocument.class).eq(OaQjdDocument::getDocNo, docNo));
                break;
            case "JBD":
                oaJbdDocumentMapper.delete(Wrappers.lambdaQuery(OaJbdDocument.class).eq(OaJbdDocument::getDocNo, docNo));
                break;
        }
        oaDocumentMapper.delete(Wrappers.lambdaQuery(OaDocument.class).eq(OaDocument::getDocNo, docNo));
    }

    // 新增单据
    @Override
    public void addDoc(JSONObject oaDocument) {

        LambdaQueryWrapper<OaFormDefinition> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(OaFormDefinition::getFieldInfo).eq(OaFormDefinition::getId, oaDocument.get("formId"));
        OaFormDefinition oaFormDefinition = oaFormDefinitionMapper.selectOne(queryWrapper);
        String sign = oaFormDefinition.getFieldInfo();

        switch (sign) {
            case "QJD":
                oaQjdDocumentService.addQjDocument(JSON.parseObject(oaDocument.toJSONString(), OaQjdDocument.class));
                break;
            case "JBD":
                oaJbdDocumentService.add(oaDocument.toJavaObject(OaJbdDocument.class));
                break;
        }
    }

    // 更新单据
    @Override
    public void updateDoc(JSONObject jsonDocument) {

        String username = SecurityUtils.getUsername();
        LocalDateTime now = LocalDateTime.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        String formattedNow = now.format(formatter);

        Map<String, Object> updateMap = jsonDocument.getInnerMap();
        String docNo = jsonDocument.get("docNo").toString();
        HashMap<String, Object> updateHashMap = new HashMap<>(updateMap);
        String tableName = (TABLE_HEAD + getFormSign(docNo) + TABLE_SUFFIX).toLowerCase();
        updateHashMap.put("tableName", tableName);
        updateHashMap.put("updateBy", username);
        updateHashMap.put("updateTime", formattedNow);

        if (StringUtils.isBlank(updateMap.get("docStatus").toString())) {
            throw new BusinessException("单据状态不能为空！");
        }

        // 更新具体单据表
        if (!updateMap.containsKey("id")) {
            throw new BusinessException("更新单据id不能为空！");
        }
        documentMapper.updateData(updateHashMap);

        // 更新单据总表
        oaDocumentMapper.update(null, Wrappers.lambdaUpdate(OaDocument.class)
                .set(OaDocument::getDocStatus, updateMap.get("docStatus"))
                .set(OaDocument::getUpdateBy, username)
                .set(OaDocument::getUpdateTime, now)
                .eq(OaDocument::getDocNo, docNo));
    }

    // 提交单据
    @Override
    public void submit(String docNo, Integer status) {
        if (status != 1) {
            throw new RuntimeException("单据提交异常，请联系管理员！");
        }
        String formSign = getFormSign(docNo);

        String username = SecurityUtils.getUsername();
        LocalDateTime now = LocalDateTime.now();

        switch (formSign) {
            case "QJD":
                oaQjdDocumentMapper.update(null, Wrappers.lambdaUpdate(OaQjdDocument.class)
                        .set(OaQjdDocument::getDocStatus, status)
                        .set(OaQjdDocument::getUpdateBy, username)
                        .set(OaQjdDocument::getUpdateTime, now)
                        .eq(OaQjdDocument::getDocNo, docNo));
                break;
            case "JBD":
                oaJbdDocumentMapper.update(null, Wrappers.lambdaUpdate(OaJbdDocument.class)
                        .set(OaJbdDocument::getDocStatus, status)
                        .set(OaJbdDocument::getUpdateBy, username)
                        .set(OaJbdDocument::getUpdateTime, now)
                        .eq(OaJbdDocument::getDocNo, docNo));
                break;
        }
        oaDocumentMapper.update(null, Wrappers.lambdaUpdate(OaDocument.class)
                .set(OaDocument::getDocStatus, status)
                .set(OaDocument::getUpdateBy, username)
                .set(OaDocument::getUpdateTime, now)
                .eq(OaDocument::getDocNo, docNo));
    }

    // 获取单据类型
    private String getFormSign(String docNo) {
        if (StringUtils.isBlank(docNo)) {
            throw new RuntimeException("单据号不能为空！");
        } else if (docNo.split("-").length != 2 ||
                docNo.split("-")[0].length() > 4 ||
                docNo.split("-")[0].length() < 3 ||
                docNo.split("-")[1].length() != 12) {
            throw new RuntimeException("单据号格式错误！");
        }
        return docNo.split("-")[0];
    }

}
